@@ -23,7 +23,9 @@ int main(int, char *argv[])
 {
   try
   {
-    const string TestStr = "Nasty\n\030Test\n\t String\r\0 With Trailer\\\\\0";
+    const string TestStr =
+      "Nasty\n\030Test\n\t String with \200\277 weird bytes "
+      "\r\0 and Trailer\\\\\0";
 
     connection C(argv[1]);
     work T(C, "test62");
@@ -55,7 +57,7 @@ int main(int, char *argv[])
       if (c == B.end())
 	throw logic_error("Premature end to binary string at " + to_string(i));
 
-      if (B.data()[i] != TestStr.at(i))
+      if (char(B.data()[i]) != TestStr.at(i))
 	throw logic_error("Binary string byte " + to_string(i) + " "
 	    "got changed from '" + to_string(char(TestStr[i])) + "' "
 	    "to '" + to_string(char(B.data()[i])) + "'");
@@ -79,7 +81,7 @@ int main(int, char *argv[])
       if (r == B.rend())
 	throw logic_error("Premature rend to binary string at " + to_string(i));
 
-      if (B[i-1] != TestStr.at(i-1))
+      if (char(B[i-1]) != char(TestStr.at(i-1)))
 	throw logic_error("Reverse iterator differs at " + to_string(i));
     }
     if (r != B.rend())
